@@ -3,7 +3,10 @@
 from __future__ import unicode_literals
 
 from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes import generic
+try:
+    from django.contrib.contenttypes.fields import GenericForeignKey
+except ImportError:  # Django < 1.9
+    from django.contrib.contenttypes.generic import GenericForeignKey
 from django.conf import settings
 from django.core import serializers
 from django.core.exceptions import ObjectDoesNotExist
@@ -46,7 +49,7 @@ class Revision(models.Model):
     """A group of related object versions."""
 
     manager_slug = models.CharField(
-        max_length = 200,
+        max_length = 191,
         db_index = True,
         default = "default",
     )
@@ -95,6 +98,10 @@ class Revision(models.Model):
         """Returns a unicode representation."""
         return ", ".join(force_text(version) for version in self.version_set.all())
 
+    #Meta
+    class Meta:
+        app_label = 'reversion'
+
 
 def has_int_pk(model):
     """Tests whether the given model has an integer primary key."""
@@ -130,7 +137,7 @@ class Version(models.Model):
                                      help_text="Content type of the model under version control.")
 
     # A link to the current instance, not the version stored in this Version!
-    object = generic.GenericForeignKey()
+    object = GenericForeignKey()
 
     format = models.CharField(max_length=255,
                               help_text="The serialization format used by this model.")
@@ -162,7 +169,9 @@ class Version(models.Model):
                 result[field.name] = field.value_from_object(obj)
             result.update(object_version.m2m_data)
             # Add parent data.
-            for parent_class, field in obj._meta.parents.items():
+            for parent_class, field in obj._meta.concrete_model._meta.parents.items():
+                if obj._meta.proxy and parent_class == obj._meta.concrete_model:
+                    continue
                 content_type = ContentType.objects.get_for_model(parent_class)
                 if field:
                     parent_id = force_text(getattr(obj, field.attname))
@@ -187,6 +196,10 @@ class Version(models.Model):
         """Returns a unicode representation."""
         return self.object_repr
 
+    #Meta
+    class Meta:
+        app_label = 'reversion'
+
 
 # Version management signals.
 pre_revision_commit = Signal(providing_args=["instances", "revision", "versions"])
